=== modified file 'drizzled/alter_drop.h'
--- drizzled/alter_drop.h	2010-02-04 08:14:46 +0000
+++ drizzled/alter_drop.h	2010-07-27 00:49:32 +0000
@@ -34,7 +34,8 @@
   enum drop_type
   {
     KEY,
-    COLUMN
+    COLUMN,
+    FOREIGN_KEY
   };
   const char *name;
   enum drop_type type;

=== modified file 'drizzled/foreign_key.cc'
--- drizzled/foreign_key.cc	2010-07-26 18:50:01 +0000
+++ drizzled/foreign_key.cc	2010-07-27 00:49:32 +0000
@@ -45,6 +45,17 @@
   message::Table::ForeignKeyConstraint *pfkey= table_message->add_fk_constraint();
   if (fkey_name)
     pfkey->set_name(fkey_name);
+  else if (table_message->has_name())
+  {
+    std::string name(table_message->name());
+    char number[20];
+
+    name.append("_ibfk_");
+    snprintf(number, sizeof(number), "%d", table_message->fk_constraint_size());
+    name.append(number);
+
+    pfkey->set_name(name);
+  }
 
   pfkey->set_match(match_opt_arg);
   pfkey->set_update_option(update_opt_arg);

=== modified file 'drizzled/sql_table.cc'
--- drizzled/sql_table.cc	2010-07-26 18:50:01 +0000
+++ drizzled/sql_table.cc	2010-07-27 00:49:32 +0000
@@ -878,7 +878,18 @@
       fk_key_count++;
       if (((Foreign_key *)key)->validate(alter_info->create_list))
         return true;
+
       Foreign_key *fk_key= (Foreign_key*) key;
+
+      add_foreign_key_to_table_message(&create_proto,
+                                       fk_key->name.str,
+                                       fk_key->columns,
+                                       fk_key->ref_table,
+                                       fk_key->ref_columns,
+                                       fk_key->delete_opt,
+                                       fk_key->update_opt,
+                                       fk_key->match_opt);
+
       if (fk_key->ref_columns.elements &&
 	  fk_key->ref_columns.elements != fk_key->columns.elements)
       {

=== modified file 'drizzled/sql_yacc.yy'
--- drizzled/sql_yacc.yy	2010-07-26 18:50:01 +0000
+++ drizzled/sql_yacc.yy	2010-07-27 00:49:32 +0000
@@ -1402,15 +1402,6 @@
                                       statement->fk_update_opt,
                                       statement->fk_match_option);
 
-	    add_foreign_key_to_table_message(&(statement->create_table_message),
-                                             $4.str ? $4.str : $1.str,
-                                             lex->col_list,
-                                             $8,
-                                             lex->ref_list,
-                                             statement->fk_delete_opt,
-                                             statement->fk_update_opt,
-                                             statement->fk_match_option);
-
             statement->alter_info.key_list.push_back(key);
             key= new Key(Key::MULTIPLE, $1.str ? $1 : $4,
                          &default_key_create_info, 1,
@@ -2266,7 +2257,8 @@
         | DROP FOREIGN KEY_SYM opt_ident
           {
             statement::AlterTable *statement= (statement::AlterTable *)Lex->statement;
-
+            statement->alter_info.drop_list.push_back(new AlterDrop(AlterDrop::FOREIGN_KEY,
+                                                                    $4.str));
             statement->alter_info.flags.set(ALTER_DROP_INDEX);
             statement->alter_info.flags.set(ALTER_FOREIGN_KEY);
           }

=== modified file 'drizzled/statement/alter_table.cc'
--- drizzled/statement/alter_table.cc	2010-06-28 12:22:48 +0000
+++ drizzled/statement/alter_table.cc	2010-07-27 00:49:32 +0000
@@ -533,15 +533,53 @@
       new_key_list.push_back(key);
     }
   }
+
+  /* Copy over existing foreign keys */
+  for (int j= 0; j < original_proto.fk_constraint_size(); j++)
+  {
+    AlterDrop *drop;
+    drop_it.rewind();
+    while ((drop= drop_it++))
+    {
+      if (drop->type == AlterDrop::FOREIGN_KEY &&
+          ! my_strcasecmp(system_charset_info, original_proto.fk_constraint(j).name().c_str(), drop->name))
+      {
+        break;
+      }
+    }
+    if (drop)
+    {
+      drop_it.remove();
+      continue;
+    }
+
+    message::Table::ForeignKeyConstraint *pfkey= table_message.add_fk_constraint();
+    *pfkey= original_proto.fk_constraint(j);
+  }
+
   {
     Key *key;
     while ((key= key_it++)) /* Add new keys */
     {
-      if (key->type == Key::FOREIGN_KEY &&
-          ((Foreign_key *)key)->validate(new_create_list))
-        goto err;
+      if (key->type == Key::FOREIGN_KEY)
+      {
+        if (((Foreign_key *)key)->validate(new_create_list))
+          goto err;
+
+        Foreign_key *fkey= (Foreign_key*)key;
+        add_foreign_key_to_table_message(&table_message,
+                                         fkey->name.str,
+                                         fkey->columns,
+                                         fkey->ref_table,
+                                         fkey->ref_columns,
+                                         fkey->delete_opt,
+                                         fkey->update_opt,
+                                         fkey->match_opt);
+      }
+
       if (key->type != Key::FOREIGN_KEY)
         new_key_list.push_back(key);
+
       if (key->name.str && is_primary_key_name(key->name.str))
       {
         my_error(ER_WRONG_NAME_FOR_INDEX,
@@ -552,6 +590,23 @@
     }
   }
 
+  /* Fix names of foreign keys being added */
+  for (int j= 0; j < table_message.fk_constraint_size(); j++)
+  {
+    if (! table_message.fk_constraint(j).has_name())
+    {
+      std::string name(table->getMutableShare()->getTableName());
+      char number[20];
+
+      name.append("_ibfk_");
+      snprintf(number, sizeof(number), "%d", j+1);
+      name.append(number);
+
+      message::Table::ForeignKeyConstraint *pfkey= table_message.mutable_fk_constraint(j);
+      pfkey->set_name(name);
+    }
+  }
+
   if (alter_info->drop_list.elements)
   {
     my_error(ER_CANT_DROP_FIELD_OR_KEY,

